-----------------------------------------------------------------------------
--
-- Tag				:= { Statement } @
-- Statement		:= literal | "[" RValue "]"
-- RValue			:= Relation
-- Relation			:= Expression [ ("==" | "~=" | ">" | "<" | "<=" | ">=") Expression ]
-- Expression		:= Term [ ("+" | "-") Term ]
-- Factor			:= Chain [ ("*" | "/") Chain ]
-- Chain			:= Primary { ":" Primary }
-- Primary			:= ConstantNumber | ConstantString | Identifier ["(" ParameterList ")"] | ( RValue )
-- ParameterList	:= RValue {, RValue }
--
-----------------------------------------------------------------------------

Aloft.TagData = { TagError = function(x) Aloft:Print(x) end, TagDebug = function(x) Aloft:Debug(x) end }

-----------------------------------------------------------------------------

Aloft.TagData.angle = 
{
	method = function(aloftData, value) return value and value ~= "" and ("<%s>"):format(value) end,
	noGuaranteeChange = true
}

Aloft.TagData.bracket =
{
	method = function(aloftData, value) return value and value ~= "" and ("(%s)"):format(value) end,
	noGuaranteeChange = true
}

Aloft.TagData.brace = 
{
	method = function(aloftData, value)	return value and value ~= "" and ("{%s}"):format(value) end,
	noGuaranteeChange = true
}

Aloft.TagData.color =
{
	method = function(aloftData, value, r, g, b) return value and ("|cff%02x%02x%02x%s|r"):format(r*255, g*255, b*255, value) end,
	noGuaranteeChange = true
}

Aloft.TagData.colorif = 
{
	method = function(aloftData, value, condition, r, g, b)
				return value and condition and ("|cff%02x%02x%02x%s|r"):format(r*255, g*255, b*255, value) or value
			 end,
	noGuaranteeChange = true
}

Aloft.TagData.format =
{
	method = function(aloftData, value, format)	return value and format:format(value) end,
	noGuaranteeChange = true,
	parameterTypes = { "string" }
}

Aloft.TagData.healthbardeficit = 
{
	events = "Aloft:OnHealthBarValueChanged",
	compileString = function(dataTable, prior) return string.format("%s%s(data.healthBarMaxValue - data.healthBarValue)", prior or "", prior and " and " or "") end,
	guaranteeResult = true,
	numeric = true
}

Aloft.TagData.healthbarmaxvalue = 
{
	events = "Aloft:OnHealthBarValueChanged",
	tableData = "healthBarMaxValue",
	guaranteeResult = true,
	numeric = true
}

Aloft.TagData.healthbarvalue = 
{
	events = "Aloft:OnHealthBarValueChanged",
	tableData = "healthBarValue",
	guaranteeResult = true,
	numeric = true
}

Aloft.TagData.healthpercent = 
{
	events = "Aloft:OnHealthBarValueChanged",
	compileString = function(dataTable, prior) return string.format("%s%sfloor(100*data.healthBarValue / data.healthBarMaxValue)", prior or "", prior and " and " or "") end,
	guaranteeResult = true,
	numeric = true
}

Aloft.TagData.isgroup =
{
	compileString = function(dataTable, prior) return string.format("data.healthBarMaxValue ~= 100%s%s", prior and " and " or "", prior or "") end,
	noGuaranteeChange = true
}

Aloft.TagData.isnotgroup = 
{
	compileString = function(dataTable, prior) return string.format("data.healthBarMaxValue == 100%s%s", prior and " and " or "", prior or "") end,
	noGuaranteeChange = true
}

Aloft.TagData.hexcolor =
{
	method = function(aloftData, value, colorValue)	return value and ("|cff%s%s|r"):format(colorValue, value or "") end,
	noGuaranteeChange = true,
	parameterTypes = { "string" }
}

Aloft.TagData.hexcolorif = 
{
	method = function(aloftData, value, condition, colorValue) return value and condition and ("|cff%s%s|r"):format(colorValue, value) or value end,
	noGuaranteeChange = true,
	parameterTypes = { "raw", "string" }
}

Aloft.TagData.hideif =
{
	compileString = function(dataTable, prior, condition)
						return string.format("not (%s)%s%s", condition, prior and " and " or "", prior or "")
					end,
	parameterTypes = { "raw" }
}

Aloft.TagData.hidezero = 
{
	method = function(aloftData, value) return value ~= 0 and value end
}

Aloft.TagData.isboss = { tableData = "isBoss" }

Aloft.TagData.isfriendly =
{
	compileString = function(dataTable, prior)
						return string.format("(data.type == \"friendlyPlayer\" or data.type == \"friendlyNPC\")%s%s", prior and " and " or "", prior or "")
					end
}

Aloft.TagData.isfriendlynpc =
{
	compileString = function(dataTable, prior) return string.format("data.type == \"friendlyNPC\"%s%s", prior and " and " or "", prior or "") end
}

Aloft.TagData.isfriendlyplayer =
{
	compileString = function(dataTable, prior) return string.format("data.type == \"friendlyPlayer\"%s%s", prior and " and " or "", prior or "") end
}

Aloft.TagData.ishostile =
{
	compileString = function(dataTable, prior) return string.format("data.type == \"hostile\"%s%s", prior and " and " or "", prior or "") end
}

Aloft.TagData.isneutral =
{
	compileString = function(dataTable, prior) return string.format("data.type == \"neutral\"%s%s", prior and " and " or "", prior or "") end
}

Aloft.TagData.ismaxhealth =
{
	events = "Aloft:OnHealthBarValueChanged",
	compileString = function(dataTable, prior) return string.format("data.healthBarValue == data.healthBarMaxValue%s%s", prior and " and " or "", prior or "") end
}

Aloft.TagData.level = { tableData = "level", numeric = true }

Aloft.TagData.lower =
{
	method = function(aloftData, value)
				if type(value) == "string" then
					return value:lower()
				end
				return value
			 end
}

Aloft.TagData.name = { tableData = "name", guaranteeResult = true }

Aloft.TagData.negate =
{
	method = function(aloftData, value)
				if type(value) == "number" then
					return -value
				end
				return value
			 end,
	noGuaranteeChange = true
}

Aloft.TagData["not"] =
{
	compileString = function(dataTable, prior, condition)
						if prior then
							return string.format("not (%s) and %s", condition, prior)
						else
							return string.format("not (%s)", condition)
						end
					end,
	parameterTypes = { "raw" }
}

Aloft.TagData.percent =
{
	method = function(aloftData, value)
				if type(value) == "number" then
					return value.."%"
				end
				return value
			 end,
	noGuaranteeChange = true
}

Aloft.TagData.prefix =
{
	method = function(aloftData, value, prefix)	return value and prefix..value end,
	noGuaranteeChange = true,
	parameterTypes = { "string" }
}

Aloft.TagData.surround =
{
	method = function(aloftData, value, prefix, suffix)	return value and prefix..value..suffix end,
	noGuaranteeChange = true,
	parameterTypes = { "string", "string" }
}

Aloft.TagData.rgb =
{
	method = function(aloftData, value, r, g, b) return value and ("|cff%02x%02x%02x%s|r"):format(r, g, b, value) end,
	parameterTypes = { "number", "number", "number" }
}

Aloft.TagData.rgbif =
{
	method = function(aloftData, value, condition, r, g, b)	return value and condition and ("|cff%02x%02x%02x%s|r"):format(r, g, b, value) or value end,
	parameterTypes = { "raw", "number", "number", "number" }
}

Aloft.TagData.short =
{
	method = function(aloftData, value)
				if type(value) == "number" then
					-- Use a 'binary' tree to determine our range, rather than the old sequential search
					-- This gives us maximum 3 comparisons
					if value <= -10000 then
						if value <= -100000 then
							if value <= -1000000 then
								return ("%.2fm"):format(value / 1000000)
							else
								return ("%dk"):format(value / 1000)
							end
						else
							return ("%.1fk"):format(value / 1000)
						end
					else
						if value < 100000 then
							if value < 10000 then
								return ("%d"):format(value)
							else
								return ("%.1fk"):format(value / 1000)
							end
						else
							if value < 1000000 then
								return ("%dk"):format(value / 1000)
							else
								return ("%.2fm"):format(value / 1000000)
							end				
						end
					end
				end
				return value
			 end,
	noGuaranteeChange = true
}

Aloft.TagData.suffix =
{
	method = function(aloftData, value, suffix)	return value and value..suffix end,
	noGuaranteeChange = true,
	parameterTypes = { "string" }
}

Aloft.TagData.upper =
{
	method = function(aloftData, value)
				if type(value) == "string" then
					return value:upper()
				end
				return value
			 end,
	noGuaranteeChange = true
}

Aloft.TagData.white	= 
{ 
	method = function(aloftData, value) return value and ("|r%s"):format(value) end,
	noGuaranteeChange = true
}

Aloft.TagData.red 		= { string = [[hexColor("ff0000")]] }
Aloft.TagData.green 	= { string = [[hexColor("00ff00")]] }
Aloft.TagData.blue 		= { string  =[[hexColor("0000ff")]] }
Aloft.TagData.cyan 		= { string = [[hexColor("00ffff")]] }
Aloft.TagData.fuchsia 	= { string = [[hexColor("ff00ff")]] }
Aloft.TagData.yellow 	= { string = [[hexColor("ffff00")]] }
Aloft.TagData.gray 		= { string = [[hexColor("afafaf")]] }
Aloft.TagData.difficultycolor =
{
	method = function(aloftData, value) return value and ("|cff%02x%02x%02x%s|r"):format(aloftData.levelTextR*255, aloftData.levelTextG*255, aloftData.levelTextB*255, value) end,
	noGuaranteeChange = true
}

-----------------------------------------------------------------------------

-- Initialise the compile cache with an empty tag, something which would otherwise need some extra special-cased code
local compileCache =
{
	[""] = { data = { }, events = { }, tag = "", method = function() return "" end },
}

function Aloft:CreateTag(tag)
	parseTag = tag or ""

	if compileCache[parseTag] then
		return compileCache[parseTag]
	end

	local result = TagCompiler:CreateTag(tag, Aloft.TagData)
	compileCache[tag] = result
	return result
end

-----------------------------------------------------------------------------
